@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+import myhdl
 from myhdl import *
 
 def bug_3529686(clr, clk, run, ack, serialout):
