@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+import myhdl
 from myhdl import *
 from myhdl.conversion import verify
 
@@ -21,7 +23,7 @@ def dut():
         d.next = 0
         for i in range(100):
             yield delay(10)
-            print rx
+            print(rx)
             a.next = (a + 37) % 256
             b.next = (b + 67) % 256
             c.next = (c + 97) % 256
