@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+import myhdl
 from myhdl import *
 from myhdl.conversion import verify
 
@@ -10,7 +12,7 @@ def dut():
         count.next = 50
         for i in range(300):
             yield delay(10)
-            print count
+            print(count)
             if count-1 < 0:
                count.next = 97
             else:
