@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+import myhdl
 from myhdl import *
 
 INT_CONDITION_0 = 0 
