@@ -1,5 +1,8 @@
+from __future__ import absolute_import
+import myhdl
 from myhdl import *
 
+@block
 def adapter(o_err, i_err, o_spec, i_spec):
 
     nomatch = Signal(bool(0))
@@ -33,15 +36,16 @@ def adapter(o_err, i_err, o_spec, i_spec):
     return assign
 
 
-def bench_adapter(conv=False):
+@block
+def bench_adapter(hdl=None):
     o_spec = ('c', 'a', 'other', 'nomatch')
     i_spec = { 'a' : 1, 'b' : 2, 'c' : 0, 'd' : 3, 'e' : 4, 'f' : 5, }
 
     o_err = Signal(intbv(0)[4:])
     i_err = Signal(intbv(0)[6:])
-    
-    if conv:
-        dut = conv(adapter, o_err, i_err, o_spec, i_spec)
+
+    if hdl:
+        dut = adapter(o_err, i_err, o_spec, i_spec).convert(hdl=hdl)
     else:
         dut = adapter(o_err, i_err, o_spec, i_spec)
 
@@ -55,13 +59,13 @@ def bench_adapter(conv=False):
             assert o_err[1] == (i_err[2] | i_err[3] | i_err[4] | i_err[5])
             assert o_err[2] == i_err[1]
             assert o_err[3] == i_err[0]
-            print o_err
+            print(o_err)
 
     return dut, stimulus
 
 def test_adapter():
-    assert conversion.verify(bench_adapter) == 0
+    assert bench_adapter().verify_convert() == 0
 
 
-bench_adapter(toVerilog)
-bench_adapter(toVHDL)
+bench_adapter('Verilog')
+bench_adapter('VHDL')
