@@ -1,9 +1,12 @@
+from __future__ import absolute_import
+import myhdl
 from myhdl import *
 from myhdl import ConversionError
 from myhdl.conversion._misc import _error
 from myhdl.conversion import verify
 
 
+@block
 def sigAugmAssignUnsupported(z, a):
     @always(a)
     def logic():
@@ -14,12 +17,13 @@ def test_SigAugmAssignUnsupported():
     z = Signal(intbv(0)[8:])
     a = Signal(intbv(0)[8:])
     try:
-        verify(sigAugmAssignUnsupported, z, a)
-    except ConversionError, e:
+        verify(sigAugmAssignUnsupported(z, a))
+    except ConversionError as e:
         assert e.kind == _error.NotSupported
     else:
         assert False
-        
+
+@block
 def modbvRange(z, a, b):
     @always(a, b)
     def logic():
@@ -33,12 +37,13 @@ def test_modbvRange():
     a = Signal(intbv(0)[4:])
     b = Signal(intbv(0)[4:])
     try:
-        verify(modbvRange, z, a, b)
-    except ConversionError, e:
+        verify(modbvRange(z, a, b))
+    except ConversionError as e:
         assert e.kind == _error.ModbvRange
     else:
         assert False
-        
+
+@block
 def modbvSigRange(z, a, b):
     @always(a, b)
     def logic():
@@ -50,13 +55,8 @@ def test_modbvSigRange():
     a = Signal(intbv(0)[4:])
     b = Signal(intbv(0)[4:])
     try:
-        verify(modbvSigRange, z, a, b)
-    except ConversionError, e:
+        verify(modbvSigRange(z, a, b))
+    except ConversionError as e:
         assert e.kind == _error.ModbvRange
     else:
         assert False
-        
-
-
-
-
