@@ -1,9 +1,12 @@
+from __future__ import absolute_import
 import os
 path = os.path
 import unittest
 
+import myhdl
 from myhdl import *
 
+@block
 def ternary1(dout, clk, rst):
 
     @always(clk.posedge, rst.negedge)
@@ -16,6 +19,7 @@ def ternary1(dout, clk, rst):
     return logic
 
 
+@block
 def ternary2(dout, clk, rst):
 
     dout_d = Signal(intbv(0)[len(dout):])
@@ -33,6 +37,7 @@ def ternary2(dout, clk, rst):
 
     return logic, comb
 
+@block
 def TernaryBench(ternary):
 
     dout = Signal(intbv(0)[8:])
@@ -54,7 +59,7 @@ def TernaryBench(ternary):
             clk.next = 1
             yield delay(10)
             assert dout == (i + 1) % 128
-            print dout
+            print(dout)
             clk.next = 0
             yield delay(10)
 
@@ -64,10 +69,12 @@ def TernaryBench(ternary):
 
 
 
-
+# uncomment when we have a VHDL-2008 compliant simulator
 def test_ternary1():
-    assert conversion.verify(TernaryBench, ternary1) == 0
+    toVHDL.name = 'ternary1'
+    assert conversion.verify(TernaryBench(ternary1)) == 0
 
 def test_ternary2():
-    assert conversion.verify(TernaryBench, ternary2) == 0
+    toVHDL.name = 'ternary2'
+    assert conversion.verify(TernaryBench(ternary2)) == 0
 
