@@ -1,7 +1,9 @@
+from __future__ import absolute_import
 import os
 path = os.path
 from random import randrange
 
+import myhdl
 from myhdl import *
 from myhdl.conversion import verify, analyze
 from myhdl import ConversionError
@@ -33,7 +35,7 @@ def LoopBench(LoopTest):
         for i in range(100):
             a.next = data[i]
             yield delay(10)
-            print z
+            print(z)
 
     return stimulus, looptest_inst
 
@@ -41,7 +43,7 @@ def LoopBench(LoopTest):
 def testForLoopError1():
     try:
         analyze(LoopBench, ForLoopError1)
-    except ConversionError, e:
+    except ConversionError as e:
         assert e.kind == _error.Requirement
     else:
         assert False
