@@ -1,17 +1,19 @@
+from __future__ import absolute_import
 import os
 path = os.path
 import unittest
 
+import myhdl
 from myhdl import *
 from myhdl import ConversionError
 from myhdl.conversion._misc import _error
 
 class TestNotSupported(unittest.TestCase):
-    
+
     def check(self, *args):
         try:
             i = toVerilog(*args)
-        except ConversionError, e:
+        except ConversionError as e:
             self.assertEqual(e.kind, _error.NotSupported)
         except:
             self.fail()
@@ -21,18 +23,6 @@ class TestNotSupported(unittest.TestCase):
     def nocheck(self, *args):
         i = toVerilog(*args)
 
-    def testAssAttr(self):
-        a = Signal(bool())
-        z = Signal(bool())
-        def g(z, a):
-            @instance
-            def logic():
-                while 1:
-                    yield a
-                    z.net = 1
-            return logic
-        self.check(g, z, a)
-
     def testAssList(self):
         a = Signal(bool())
         z = Signal(bool())
@@ -45,7 +35,7 @@ class TestNotSupported(unittest.TestCase):
                     [p, q] = 1, 2
             return logic
         self.check(g, z, a)
-            
+
     def testAssTuple(self):
         a = Signal(bool())
         z = Signal(bool())
@@ -59,20 +49,6 @@ class TestNotSupported(unittest.TestCase):
             return logic
         self.check(g, z, a)
 
-    def testBackquote(self):
-        a = Signal(bool())
-        z = Signal(bool())
-        def g(z, a):
-            @instance
-            def logic():
-                while 1:
-                    yield a
-                    z.next = 1
-                    `a`
-            return logic
-        self.check(g, z, a)
-            
-       
     def testClass(self):
         a = Signal(bool())
         z = Signal(bool())
@@ -87,7 +63,7 @@ class TestNotSupported(unittest.TestCase):
             return logic
         self.check(g, z, a)
 
-        
+
     def testDict(self):
         a = Signal(bool())
         z = Signal(bool())
@@ -113,19 +89,6 @@ class TestNotSupported(unittest.TestCase):
             return logic
         self.check(g, z, a)
 
-    def testExec(self):
-        a = Signal(bool())
-        z = Signal(bool())
-        def g(z, a):
-            @instance
-            def logic():
-                while 1:
-                    yield a
-                    z.next = 1
-                    exec "1 + 2" in globals , locals
-            return logic
-        self.check(g, z, a)
-        
     def testFrom(self):
         a = Signal(bool())
         z = Signal(bool())
@@ -236,7 +199,7 @@ class TestNotSupported(unittest.TestCase):
 ##                 z.next = 1
 ##                 return
 ##         self.check(g, z, a)
-        
+
     def testTryExcept(self):
         a = Signal(bool())
         z = Signal(bool())
@@ -292,7 +255,7 @@ class TestNotSupported(unittest.TestCase):
 ##                 else:
 ##                     z.next = a and b
 ##         self.check(g, z, a, b)
-    
+
 ##     def testShortcutOr(self):
 ##         a, b, c  = [Signal(bool()) for i in range(3)]
 ##         z = Signal(bool())
@@ -320,7 +283,7 @@ class TestNotSupported(unittest.TestCase):
                         z.next = a and b
             return logic
         self.check(g, z, a, b)
-    
+
     def testNonBoolArgOr(self):
         a = Signal(bool())
         b = intbv(0)[2:]
@@ -366,7 +329,7 @@ class TestNotSupported(unittest.TestCase):
         def f(a, b, c):
             return g(a)
         x = self.check(f, a, b, c)
-        
+
     def testExtraNamedArgsInCall(self):
         a, b, c = [Signal(bool()) for i in range(3)]
         c = [1]
@@ -382,7 +345,7 @@ class TestNotSupported(unittest.TestCase):
         def f(a, b, c):
             return g(a)
         x = self.check(f, a, b, c)
-   
+
 
 
 class TestMisc(unittest.TestCase):
@@ -402,11 +365,11 @@ class TestMisc(unittest.TestCase):
             return g(a)
         f(a, b, c)
         x = toVerilog(f, a, b, c)
-    
-        
+
+
 
 
 if __name__ == '__main__':
     unittest.main()
-    
+
 
