@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 import os
 path = os.path
 import unittest
@@ -6,9 +7,10 @@ import random
 from random import randrange
 random.seed(2)
 
+import myhdl
 from myhdl import *
 
-from util import setupCosimulation
+from .util import setupCosimulation
 
 
 def binaryOps(
@@ -146,7 +148,7 @@ class TestBinaryOps(TestCase):
                            And,
                            Or,
                            left, right)
-        binops_v = binaryOps_v(binaryOps.func_name,
+        binops_v = binaryOps_v(binaryOps.__name__,
                                Bitand_v,
                                Bitor_v,
                                Bitxor_v,
@@ -274,7 +276,7 @@ class TestMultiOps(TestCase):
                            And,
                            Or,
                            argm, argn, argp)
-        multiops_v = multiOps_v(multiOps.func_name,
+        multiops_v = multiOps_v(multiOps.__name__,
                                 Bitand_v,
                                 Bitor_v,
                                 Bitxor_v,
@@ -371,7 +373,7 @@ class TestUnaryOps(TestCase):
                              UnaryAdd,
                              UnarySub,
                              arg)
-        unaryops_v = unaryOps_v(unaryOps.func_name,
+        unaryops_v = unaryOps_v(unaryOps.__name__,
                                 Not_v,
                                 Invert_v,
                                 UnaryAdd_v,
@@ -514,7 +516,7 @@ class TestAugmOps(TestCase):
                            Sub,
                            Sum,
                            left, right)
-        augmops_v = augmOps_v( augmOps.func_name,
+        augmops_v = augmOps_v( augmOps.__name__,
                                Bitand_v,
                                Bitor_v,
                                Bitxor_v,
