@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 import os
 path = os.path
 import unittest
@@ -6,9 +7,10 @@ import random
 from random import randrange
 random.seed(2)
 
+import myhdl
 from myhdl import *
 
-from util import setupCosimulation
+from .util import setupCosimulation
 
 
 def binaryOps(
@@ -150,7 +152,7 @@ class TestBinaryOps(TestCase):
                            And,
                            Or,
                            left, right, bit)
-        binops_v = binaryOps_v(binaryOps.func_name,
+        binops_v = binaryOps_v(binaryOps.__name__,
 ##                                Bitand_v,
 ##                                Bitor_v,
 ##                                Bitxor_v,
@@ -295,7 +297,7 @@ class TestUnaryOps(TestCase):
                              UnaryAdd,
                              UnarySub,
                              arg)
-        unaryops_v = unaryOps_v(unaryOps.func_name,
+        unaryops_v = unaryOps_v(unaryOps.__name__,
                                 Not_v,
                                 Invert_v,
                                 UnaryAdd_v,
@@ -448,7 +450,7 @@ class TestAugmOps(TestCase):
                            Sub,
                            Sum,
                            left, right)
-        augmops_v = augmOps_v( augmOps.func_name,
+        augmops_v = augmOps_v( augmOps.__name__,
 ##                                Bitand_v,
 ##                                Bitor_v,
 ##                                Bitxor_v,
