@@ -1,6 +1,8 @@
+from __future__ import absolute_import
 import os
 path = os.path
-
+import subprocess
+import myhdl
 from myhdl import *
 
 # Icarus
@@ -9,9 +11,9 @@ def setupCosimulationIcarus(**kwargs):
     objfile = "%s.o" % name
     if path.exists(objfile):
         os.remove(objfile)
-    analyze_cmd = "iverilog -o %s %s.v tb_%s.v" % (objfile, name, name)
-    os.system(analyze_cmd)
-    simulate_cmd = "vvp -m ../../../../cosimulation/icarus/myhdl.vpi %s" % objfile
+    analyze_cmd = ['iverilog', '-o', objfile, '%s.v' %name, 'tb_%s.v' % name]
+    subprocess.call(analyze_cmd)
+    simulate_cmd = ['vvp', '-m', '../../../../cosimulation/icarus/myhdl.vpi', objfile]
     return Cosimulation(simulate_cmd, **kwargs)
 
 # cver
