@@ -18,46 +18,41 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 """ Run unit tests for Simulation """
+from __future__ import absolute_import
 
-
-import unittest
-from unittest import TestCase
 import random
 from random import randrange
-random.seed(1) # random, but deterministic
+from unittest import TestCase
 
-from myhdl import Simulation, SimulationError, now, delay, StopSimulation, join
-from myhdl import Signal, intbv
+from myhdl import (Signal, Simulation, SimulationError, StopSimulation, delay,
+                   intbv, join, now)
 from myhdl._Simulation import _error
+from helpers import raises_kind
+
+random.seed(1)  # random, but deterministic
 
-from myhdl._simulator import _siglist
 
 QUIET=1
 
+
 class Shared:
     pass
 
+
 class SimArgs(TestCase):
     """ Simulation arguments """
+
     def test1(self):
-        try:
+        with raises_kind(SimulationError, _error.ArgType):
             Simulation(None)
-        except SimulationError, e:
-            self.assertEqual(e.kind, _error.ArgType)
-        except:
-            self.fail()
 
     def test2(self):
         def g():
             yield delay(10)
         i = g()
-        try:
+        with raises_kind(SimulationError, _error.DuplicatedArg):
             Simulation(i, i)
-        except SimulationError, e:
-            self.assertEqual(e.kind, _error.DuplicatedArg)
-        except:
-            self.fail()
-            
+
 
 class YieldNone(TestCase):
     """ Basic test of yield None behavior """
@@ -68,11 +63,11 @@ class YieldNone(TestCase):
             yield delay(10)
             a.next = 1
             yield None
-            self.assertEqual(a.val, 0)
-            self.assertEqual(now(), 10)
+            assert a.val == 0
+            assert now() == 10
             yield delay(0)
-            self.assertEqual(a.val, 1)
-            self.assertEqual(now(), 10)
+            assert a.val == 1
+            assert now() == 10
         Simulation(stimulus()).run(quiet=QUIET)
 
     def test2(self):
@@ -80,19 +75,19 @@ class YieldNone(TestCase):
             a = Signal(0)
             yield delay(10)
             a.next = 1
-            self.assertEqual(a.val, 0)
-            self.assertEqual(now(), 10)
+            assert a.val == 0
+            assert now() == 10
             yield None
             a.next = 0
-            self.assertEqual(a.val, 0)
-            self.assertEqual(now(), 10)
+            assert a.val == 0
+            assert now() == 10
             yield None
             a.next = 1
-            self.assertEqual(a.val, 0)
-            self.assertEqual(now(), 10)
+            assert a.val == 0
+            assert now() == 10
             yield delay(0)
-            self.assertEqual(a.val, 1)
-            self.assertEqual(now(), 10)
+            assert a.val == 1
+            assert now() == 10
         Simulation(stimulus()).run(quiet=QUIET)
 
     def test3(self):
@@ -101,49 +96,51 @@ class YieldNone(TestCase):
             yield delay(10)
             a.next = 1
             yield None, delay(10)
-            self.assertEqual(a.val, 0)
-            self.assertEqual(now(), 10)
+            assert a.val == 0
+            assert now() == 10
             yield delay(0)
-            self.assertEqual(a.val, 1)
-            self.assertEqual(now(), 10)
+            assert a.val == 1
+            assert now() == 10
         Simulation(stimulus()).run(quiet=QUIET)
 
     def test4(self):
         def stimulus():
             a = Signal(0)
             yield delay(10)
+
             def gen():
                 yield delay(20)
                 a.next = 1
             yield None, gen()
-            self.assertEqual(a.val, 0)
-            self.assertEqual(now(), 10)
+            assert a.val == 0
+            assert now() == 10
             yield delay(25)
-            self.assertEqual(a.val, 1)
+            assert a.val == 1
         Simulation(stimulus()).run(quiet=QUIET)
-            
+
 
 class JoinMix(TestCase):
     """ Test of joins mixed with other clauses """
-         
+
     def test1(self):
         def stimulus():
             a = Signal(0)
+
             def gen():
                 yield join(delay(10), delay(20))
             yield gen(), delay(5)
-            self.assertEqual(now(), 5)
+            assert now() == 5
             yield a
-            self.fail("Incorrect run") # should not get here
+            raise AssertionError("Incorrect run")  # should not get here
         Simulation(stimulus()).run(quiet=QUIET)
-        
+
     def test2(self):
         def stimulus():
             a = Signal(0)
             yield join(delay(10), delay(20)), delay(5)
-            self.assertEqual(now(), 5)
+            assert now() == 5
             yield a
-            self.fail("Incorrect run") # should not get here
+            raise AssertionError("Incorrect run")  # should not get here
         Simulation(stimulus()).run(quiet=QUIET)
 
     def stimulus(self, a, b, c, d):
@@ -161,75 +158,84 @@ class JoinMix(TestCase):
         b.next = 1
         c.next = 0
         d.next = 1
-        
+
     def test3(self):
         a, b, c, d = [Signal(0) for i in range(4)]
+
         def response():
             yield join(a, b, c, d)
-            self.assertEqual(now(), 20)
+            assert now() == 20
         Simulation(self.stimulus(a, b, c, d), response()).run(quiet=QUIET)
-        
+
     def test4(self):
         a, b, c, d = [Signal(0) for i in range(4)]
+
         def response():
             yield join(a, b), join(c, d)
-            self.assertEqual(now(), 10)
+            assert now() == 10
         Simulation(self.stimulus(a, b, c, d), response()).run(quiet=QUIET)
-        
+
     def test5(self):
         a, b, c, d = [Signal(0) for i in range(4)]
+
         def response():
             yield join(a), b, join(c, d)
-            self.assertEqual(now(), 5)
+            assert now() == 5
         Simulation(self.stimulus(a, b, c, d), response()).run(quiet=QUIET)
-        
+
     def test6(self):
         a, b, c, d = [Signal(0) for i in range(4)]
+
         def response():
             yield join(a, delay(20)), b, join(c, d)
-            self.assertEqual(now(), 10)
+            assert now() == 10
         Simulation(self.stimulus(a, b, c, d), response()).run(quiet=QUIET)
 
     def test7(self):
         a, b, c, d = [Signal(0) for i in range(4)]
+
         def response():
             yield join(a, delay(30)), join(c, d)
-            self.assertEqual(now(), 20)
+            assert now() == 20
         Simulation(self.stimulus(a, b, c, d), response()).run(quiet=QUIET)
 
     def test8(self):
         a, b, c, d = [Signal(0) for i in range(4)]
+
         def response():
             yield join(a, a.negedge)
-            self.assertEqual(now(), 10)
+            assert now() == 10
         Simulation(self.stimulus(a, b, c, d), response()).run(quiet=QUIET)
 
     def test9(self):
         a, b, c, d = [Signal(0) for i in range(4)]
+
         def response():
             yield join(a, a.negedge, c.posedge)
-            self.assertEqual(now(), 15)
+            assert now() == 15
         Simulation(self.stimulus(a, b, c, d), response()).run(quiet=QUIET)
 
     def test10(self):
         a, b, c, d = [Signal(0) for i in range(4)]
+
         def response():
             yield join(a, a)
-            self.assertEqual(now(), 5)
+            assert now() == 5
         Simulation(self.stimulus(a, b, c, d), response()).run(quiet=QUIET)
-        
+
     def test11(self):
         a, b, c, d = [Signal(0) for i in range(4)]
+
         def response():
             yield join(a, b.posedge, b.negedge, a)
-            self.assertEqual(now(), 15)
+            assert now() == 15
         Simulation(self.stimulus(a, b, c, d), response()).run(quiet=QUIET)
-          
+
 
 class JoinedGen(TestCase):
-    
+
     """ Basic test of yielding joined concurrent generators """
-    
+
     def bench(self):
         clk = Signal(0)
         sig1 = Signal(0)
@@ -250,20 +256,20 @@ class JoinedGen(TestCase):
             sig1.next = 0
             sig2.next = 0
             yield join(delay(n0*td), gen(sig1, n1), gen(sig2, n2))
-            self.assertEqual(sig1.val, 1)
-            self.assertEqual(sig2.val, 1)
-            self.assertEqual(now(), offset + td * max(n0, n1, n2))
+            assert sig1.val == 1
+            assert sig2.val == 1
+            assert now() == offset + td * max(n0, n1, n2)
 
-        raise StopSimulation, "Joined concurrent generator yield"
+        raise StopSimulation("Joined concurrent generator yield")
 
     def testYieldJoinedGen(self):
         Simulation(self.bench()).run(quiet=QUIET)
-        
+
 
 class SignalUpdateFirst(TestCase):
-    
+
     """ Check that signal updates are done first, as in VHDL """
-    
+
     def bench(self):
 
         Q = Signal(0, delay=9)
@@ -279,25 +285,25 @@ class SignalUpdateFirst(TestCase):
             R.next = 1
             S.next = 1
             yield delay(10)
-            self.assertEqual(Q.val, 1) # control
-            self.assertEqual(R.val, 1) # actual check
-            self.assertEqual(S.val, 0) # control
+            assert Q.val == 1  # control
+            assert R.val == 1  # actual check
+            assert S.val == 0  # control
             yield delay(1)
-            self.assertEqual(Q.val, 1) # control
-            self.assertEqual(R.val, 1) # control
-            self.assertEqual(S.val, 1) # control
-            raise StopSimulation, "Signal update test"
+            assert Q.val == 1  # control
+            assert R.val == 1  # control
+            assert S.val == 1  # control
+            raise StopSimulation("Signal update test")
 
         return process()
 
     def testSignalUpdateFirst(self):
         Simulation(self.bench()).run(quiet=QUIET)
-        
-        
+
+
 class YieldZeroDelay(TestCase):
-    
+
     """ Basic test of yielding a zero delay """
-    
+
     def bench(self):
         clk = Signal(0)
         sig1 = Signal(0)
@@ -313,27 +319,27 @@ class YieldZeroDelay(TestCase):
         for i in range(100):
             offset = now()
             n1 = randrange(2, 10)
-            n2 = randrange(n1+1, 20) # n2 > n1
+            n2 = randrange(n1+1, 20)  # n2 > n1
             yield delay(0), gen(sig1, n1), gen(sig2, n2)
-            self.assertEqual(sig1.val, 0)
-            self.assertEqual(sig2.val, 0)
-            self.assertEqual(now(), offset + 0)
+            assert sig1.val == 0
+            assert sig2.val == 0
+            assert now() == offset + 0
             yield sig1.posedge
-            self.assertEqual(sig2.val, 0)
-            self.assertEqual(now(), offset + n1*td)
+            assert sig2.val == 0
+            assert now() == offset + n1*td
             yield sig2.posedge
-            self.assertEqual(now(), offset + n2*td)
-        
-        raise StopSimulation, "Zero delay yield"
+            assert now() == offset + n2*td
+
+        raise StopSimulation("Zero delay yield")
 
     def testYieldZeroDelay(self):
         Simulation(self.bench()).run(quiet=QUIET)
 
 
 class YieldConcurrentGen(TestCase):
-    
+
     """ Basic test of yielding concurrent generators """
-    
+
     def bench(self):
         clk = Signal(0)
         sig1 = Signal(0)
@@ -349,28 +355,27 @@ class YieldConcurrentGen(TestCase):
         for i in range(100):
             offset = now()
             n1 = randrange(2, 10)
-            n2 = randrange(n1+1, 20) # n2 > n1
+            n2 = randrange(n1+1, 20)  # n2 > n1
             yield delay(td), gen(sig1, n1), gen(sig2, n2)
-            self.assertEqual(sig1.val, 0)
-            self.assertEqual(sig2.val, 0)
-            self.assertEqual(now(), offset + td)
+            assert sig1.val == 0
+            assert sig2.val == 0
+            assert now() == offset + td
             yield sig1.posedge
-            self.assertEqual(sig2.val, 0)
-            self.assertEqual(now(), offset + n1*td)
+            assert sig2.val == 0
+            assert now() == offset + n1*td
             yield sig2.posedge
-            self.assertEqual(now(), offset + n2*td)
+            assert now() == offset + n2*td
 
-        raise StopSimulation, "Concurrent generator yield"
+        raise StopSimulation("Concurrent generator yield")
 
     def testYieldConcurrentGen(self):
         Simulation(self.bench()).run(quiet=QUIET)
 
-        
 
 class YieldGen(TestCase):
-    
+
     """ Basic test of yielding generators """
-    
+
     def bench(self):
 
         clk = Signal(0)
@@ -387,7 +392,7 @@ class YieldGen(TestCase):
                 l.append(e)
                 expectedCnt += e
                 expected.append(expectedCnt)
-            nlists.append(l)                
+            nlists.append(l)
 
         def clkGen():
             while 1:
@@ -401,16 +406,16 @@ class YieldGen(TestCase):
             for i in range(n):
                 yield clk.posedge
                 shared.cnt += 1
-            self.assertEqual(shared.cnt, expected[shared.i])
+            assert shared.cnt == expected[shared.i]
             shared.i += 1
             if nlist:
                 yield task(nlist)
-                
+
         def module():
             for nlist in nlists:
                 yield task(nlist)
-            self.assertEqual(shared.cnt, expected[-1])
-            raise StopSimulation, "Generator yield"
+            assert shared.cnt == expected[-1]
+            raise StopSimulation("Generator yield")
 
         return(module(), clkGen())
 
@@ -419,7 +424,7 @@ class YieldGen(TestCase):
 
 
 class DeltaCycleOrder(TestCase):
-    
+
     """ Check that delta cycle order does not matter """
 
     def bench(self, function):
@@ -435,7 +440,7 @@ class DeltaCycleOrder(TestCase):
         s = [a, b, c, d]
         vectors = [intbv(j) for i in range(8) for j in range(16)]
         random.shuffle(vectors)
-        index = range(4)
+        index = list(range(4))
 
         def clkGen():
             while 1:
@@ -468,8 +473,8 @@ class DeltaCycleOrder(TestCase):
                 random.shuffle(index)
                 yield clk.posedge
                 yield clk.posedge
-                self.assertEqual(z.val, function(v[0], v[1], v[2], v[3]))
-            raise StopSimulation, "Delta cycle order"
+                assert z.val == function(v[0], v[1], v[2], v[3])
+            raise StopSimulation("Delta cycle order")
 
         inputGen = [inGen(i) for i in range(4)]
         instance = [clkGen(), deltaGen(), logic(), stimulus(), inputGen]
@@ -479,12 +484,12 @@ class DeltaCycleOrder(TestCase):
         def andFunction(a, b, c, d):
             return a & b & c & d
         Simulation(self.bench(andFunction)).run(quiet=QUIET)
-        
+
     def testOr(self):
         def orFunction(a, b, c, d):
             return a | b | c | d
         Simulation(self.bench(orFunction)).run(quiet=QUIET)
-        
+
     def testXor(self):
         def xorFunction(a, b, c, d):
             return a ^ b ^ c ^ d
@@ -502,14 +507,14 @@ class DeltaCycleOrder(TestCase):
         def function(a, b, c, d):
             return not (a & (not b)) | ((not c) & d)
         Simulation(self.bench(function)).run(quiet=QUIET)
-    
+
 
 class DeltaCycleRace(TestCase):
-    
+
     """ Check that delta cycle races are like in VHDL """
-    
+
     def bench(self):
-        
+
         uprange = range(300)
         msig = Signal(uprange[0])
         ssig = [Signal(uprange[-1]) for i in range(2)]
@@ -518,35 +523,35 @@ class DeltaCycleRace(TestCase):
         deltaClk = Signal(0)
         shared = Shared()
         shared.t = now()
-        
+
         def clkGen():
             for i in uprange[:-1]:
                 yield delay(10)
                 clk.next = 1
                 yield delay(10)
                 clk.next = 0
-                
+
         def deltaClkGen():
             while 1:
                 yield clk
                 deltaClk.next = clk.val
-            
+
         def master():
             i = 0
             while 1:
                 yield clk.posedge
                 msig.next = uprange[i+1]
-                self.assertEqual(msig.val, uprange[i])
+                assert msig.val == uprange[i]
                 shared.t = now()
                 i += 1
-                
+
         def slave(ssig):
             """ Double-check proper operation """
             i = 0
             while 1:
                 yield clk.posedge
                 ssig.next = msig.val
-                self.assertEqual(ssig.val, uprange[i-1])
+                assert ssig.val == uprange[i-1]
                 i += 1
 
         def deltaSlave(dsig):
@@ -555,15 +560,14 @@ class DeltaCycleRace(TestCase):
             while 1:
                 yield deltaClk.posedge
                 dsig.next = msig.val
-                self.assertEqual(now(), shared.t)
-                self.assertEqual(dsig.val, uprange[i])
+                assert now() == shared.t
+                assert dsig.val == uprange[i]
                 i += 1
-                
+
         return (slave(ssig[1]), deltaSlave(dsig[1]),
                 master(), clkGen(), deltaClkGen(),
                 slave(ssig[0]), deltaSlave(dsig[0]))
-    
-    
+
     def testDeltaCycleRace(self):
         """ Check delta cycle races """
         bench = self.bench()
@@ -571,32 +575,32 @@ class DeltaCycleRace(TestCase):
 
 
 class DelayLine(TestCase):
-    
+
     """ Check that delay lines work properly """
 
     def bench(self):
         uprange = range(500)
         sig_Z = [Signal(uprange[-i]) for i in range(7)]
         clk = Signal(0)
-        
+
         def clkGen():
             for i in uprange[:-1]:
                 yield delay(10)
                 clk.next = 1
                 yield delay(10)
                 clk.next = 0
-                
+
         def delayElement(n, i):
                 sig_Z[n].next = sig_Z[n-1].val
-                self.assertEqual(sig_Z[n].val, uprange[i-n])
-                
+                assert sig_Z[n].val == uprange[i-n]
+
         def stage(n):
             i = 0
             while 1:
                 yield clk.posedge
                 delayElement(n, i)
                 i += 1
-                
+
         def stage012():
             i = 0
             while 1:
@@ -605,16 +609,15 @@ class DelayLine(TestCase):
                 sig_Z[0].next = uprange[i+1]
                 delayElement(2, i)
                 i += 1
-                
+
         return [stage(6), stage(4), clkGen(), stage(3), stage012(), stage(5)]
-                   
+
     def testZeroDelay(self):
         """ Zero delay behavior """
         bench = self.bench()
         Simulation(bench).run(quiet=QUIET)
 
 
-
 def initSignal(waveform):
     interval, val, sigdelay = waveform[0]
     if sigdelay:
@@ -622,18 +625,23 @@ def initSignal(waveform):
     else:
         return Signal(val=val)
 
+
 def isPosedge(oldval, val):
     return not oldval and val
 
+
 def isNegedge(oldval, val):
     return oldval and not val
 
+
 def isEvent(oldval, val):
     return oldval != val
 
+
 def isEdge(oldval, val):
     return isPosedge(oldval, val) or isNegedge(oldval, val)
 
+
 def getExpectedTimes(waveform, eventCheck):
     interval, val, sigdelay = waveform[0]
     # print waveform[0]
@@ -652,16 +660,16 @@ def getExpectedTimes(waveform, eventCheck):
             inctime += waveform[j][0]
             newval = waveform[j][1]
             newsigdelay = waveform[j][2]
-            if newval != val: # cancel event
+            if newval != val:  # cancel event
                 break
-            else: # same vals
+            else:  # same vals
                 if inctime + newsigdelay < sigdelay:
                     # special case: there is a later event, with same val,
                     # but smaller delay: presumably, this should win,
                     # so cancel the present one
                     break
             j += 1
-        else: # if event was not cancelled by a break
+        else:  # if event was not cancelled by a break
             if eventCheck(oldval, val):
                 expected.append(time + sigdelay)
                 # print expected[-1]
@@ -672,7 +680,7 @@ def getExpectedTimes(waveform, eventCheck):
 
 
 class Waveform(TestCase):
-    
+
     """ Test of all sorts of event response in a waveform """
 
     waveform = []
@@ -683,7 +691,7 @@ class Waveform(TestCase):
         val = randrange(0, 4)
         waveform.append((interval, val, sigdelay))
         duration = interval + duration
-         
+
     def stimulus(self):
         for interval, val, sigdelay in self.waveform:
             yield delay(interval)
@@ -692,36 +700,36 @@ class Waveform(TestCase):
                 self.sig.delay = sigdelay
 
     def response(self, clause, expected):
-        self.assert_(len(expected) > 100) # we should test something
+        assert len(expected) > 100  # we should test something
         i = 0
         while 1:
             yield clause
-            self.assertEqual(now(), expected[i])
+            assert now() == expected[i]
             i += 1
-            
+
     def setUp(self):
         self.sig = initSignal(self.waveform)
 
     def runSim(self, sim):
         sim.run(quiet=QUIET)
-                 
+
     def testPosedge(self):
         """ Posedge waveform test """
         s = self.sig
         stimulus = self.stimulus()
-        expected = getExpectedTimes(self.waveform, isPosedge)     
+        expected = getExpectedTimes(self.waveform, isPosedge)
         response = self.response(clause=s.posedge, expected=expected)
         self.runSim(Simulation(stimulus, response))
-        self.assert_(self.duration <= now())
+        assert self.duration <= now()
 
     def testNegedge(self):
         """ Negedge waveform test """
         s = self.sig
         stimulus = self.stimulus()
-        expected = getExpectedTimes(self.waveform, isNegedge)     
+        expected = getExpectedTimes(self.waveform, isNegedge)
         response = self.response(clause=s.negedge, expected=expected)
         self.runSim(Simulation(stimulus, response))
-        self.assert_(self.duration <= now())
+        assert self.duration <= now()
 
     def testEdge(self):
         """ Edge waveform test """
@@ -731,58 +739,58 @@ class Waveform(TestCase):
         response = self.response(clause=(s.negedge, s.posedge),
                                  expected=expected)
         self.runSim(Simulation(stimulus, response))
-        self.assert_(self.duration <= now())
+        assert self.duration <= now()
 
     def testEvent(self):
         """ Event waveform test """
         s = self.sig
         stimulus = self.stimulus()
-        expected = getExpectedTimes(self.waveform, isEvent)     
+        expected = getExpectedTimes(self.waveform, isEvent)
         # print expected
         response = self.response(clause=s, expected=expected)
         self.runSim(Simulation(stimulus, response))
-        self.assert_(self.duration <= now())
-            
+        assert self.duration <= now()
+
     def testRedundantEvents(self):
         """ Redundant event waveform test """
         s = self.sig
         stimulus = self.stimulus()
-        expected = getExpectedTimes(self.waveform, isEvent)     
+        expected = getExpectedTimes(self.waveform, isEvent)
         response = self.response(clause=(s,) * 6, expected=expected)
         self.runSim(Simulation(stimulus, response))
-        self.assert_(self.duration <= now())
-        
-    def testRedundantEventAndEdges(self):       
+        assert self.duration <= now()
+
+    def testRedundantEventAndEdges(self):
         """ Redundant edge waveform test """
         s = self.sig
         stimulus = self.stimulus()
-        expected = getExpectedTimes(self.waveform, isEvent)     
+        expected = getExpectedTimes(self.waveform, isEvent)
         response = self.response(clause=(s, s.negedge, s.posedge),
                                  expected=expected)
         self.runSim(Simulation(stimulus, response))
-        self.assert_(self.duration <= now())
-        
+        assert self.duration <= now()
+
     def testRedundantPosedges(self):
         """ Redundant posedge waveform test """
         s = self.sig
         stimulus = self.stimulus()
-        expected = getExpectedTimes(self.waveform, isPosedge)     
+        expected = getExpectedTimes(self.waveform, isPosedge)
         response = self.response(clause=(s.posedge,) * 3, expected=expected)
         self.runSim(Simulation(stimulus, response))
-        self.assert_(self.duration <= now())
+        assert self.duration <= now()
 
     def testRedundantNegedges(self):
         """ Redundant negedge waveform test """
         s = self.sig
         stimulus = self.stimulus()
-        expected = getExpectedTimes(self.waveform, isNegedge)     
+        expected = getExpectedTimes(self.waveform, isNegedge)
         response = self.response(clause=(s.negedge,) * 9, expected=expected)
         self.runSim(Simulation(stimulus, response))
-        self.assert_(self.duration <= now())
+        assert self.duration <= now()
+
 
-        
 class WaveformSigDelay(Waveform):
-    
+
     """ Repeat waveform tests with a delayed signal """
 
     waveform = []
@@ -794,10 +802,10 @@ class WaveformSigDelay(Waveform):
         sigdelay = randrange(1, 20)
         waveform.append((interval, val, sigdelay))
         duration += interval
-        
+
 
 class WaveformInertialDelay(Waveform):
-    
+
     """ Repeat waveform tests to check inertial delay """
 
     waveform = []
@@ -810,8 +818,9 @@ class WaveformInertialDelay(Waveform):
         waveform.append((interval, val, sigdelay))
         duration += interval
 
+
 class WaveformInertialDelayStress(Waveform):
-    
+
     """ Repeat waveform tests to stress inertial delay """
 
     waveform = []
@@ -824,29 +833,32 @@ class WaveformInertialDelayStress(Waveform):
         waveform.append((interval, val, sigdelay))
         duration += interval
 
+
 class SimulationRunMethod(Waveform):
-    
+
     """ Basic test of run method of Simulation object """
-    
+
     def runSim(self, sim):
         duration = randrange(1, 300)
         while sim.run(duration, quiet=QUIET):
             duration = randrange(1, 300)
 
-      
+
 class TimeZeroEvents(TestCase):
 
     """ Check events at time 0 """
 
     def bench(self, sig, next, clause, timeout=1):
         val = sig.val
+
         def stimulus():
             sig.next = next
             yield delay(10)
+
         def response():
             yield clause, delay(timeout)
-            self.assertEqual(now(), 0)
-            self.assertEqual(sig.val, next)
+            assert now() == 0
+            assert sig.val == next
         return [stimulus(), response()]
 
     def testEvent(self):
@@ -860,15 +872,9 @@ class TimeZeroEvents(TestCase):
         s = Signal(0)
         testBench = self.bench(sig=s, next=1, clause=s.posedge)
         Simulation(testBench).run(quiet=QUIET)
-        
+
     def testNegedge(self):
         """ Negedge at time 0 """
         s = Signal(1)
         testBench = self.bench(sig=s, next=0, clause=s.negedge)
         Simulation(testBench).run(quiet=QUIET)
-
-
-        
-if __name__ == "__main__":
-    unittest.main()
-                
